/**
* Copyright (C) 2008 Seapeak.Xu / xvhfeng@gmail.com
*
* FastLib may be copied only under the terms of the GNU General
* Public License V3, which may be found in the FastLib source kit.
* Please visit the FastLib Home Page http://www.csource.org/ for more detail.
**/

#ifndef PTHREAD_POOL_H_
#define PTHREAD_POOL_H_

#include <pthread.h>

/*
* define the callback function type of thread
*/
typedef void(*callback)(void *);


/*
* the thread pool state
* member:
* 			uninitialized : not initialize the thread pool.
* 			initializing : initializing the thread pool.
* 			initialized : the pool can use.
* 			uninstalling : uninstalling the thread pool.
* 			uninstalled : uninstall the thread pool is over.
*/
typedef enum threadpool_state
{
	uninitialized,
	initializing,
	initialized,
	uninstalling,
	uninstalled,
}thread_state_t;


/*
* define the thread type which in the pool
* members:
* 			id : the thread id
* 			mutex_locker : the  mutext locker
* 			run_locker : the locker for noticing the thread do running or waitting
* 			func : the callback function for thread
* 			arg : the callback parameter
*/
typedef struct thread_info
{
	pthread_t id;
	pthread_mutex_t mutex_locker;
	pthread_cond_t run_locker;
	callback func;
	void *arg;
}thread_info_t;

/*
* the structure for the thread pool
* member:
* 			list : the initialazed thread list
* 			mutex_locker : the mutex locker for the thread operation.
* 			run_locker : the locker for noticing the thread do running or waitting.
* 			full_locker : the locker notice the thread is stoping when free the thread pool and the pool is not full .
* 			empty_Locker : the locker notice the thread waitting for the busy thread work over,then do with the thread.
* 			state : the pool's current state.
*          total_size : the pool max size;
*          current_size : the thread count for the current pool ;
*          current_index : the busy thread in the  pool index.
*/
typedef struct threadpool_info
{
	thread_info_t **list;
	pthread_mutex_t mutex_locker;
	pthread_cond_t run_locker;
	pthread_cond_t full_locker;
	pthread_cond_t empty_locker;
	thread_state_t state;
	int total_size;
	int current_size;
	int current_index;
}threadpool_info_t;

#ifdef __cplusplus
extern "C" {
#endif

	/*
	* initialize the thread pool
	* parameters:
	* 				size : thread pool max size
	* return:
	* 				0:initialize pool success;
	* 				-1:the size parameter is less 0;
	* 				-2:initialize pool is fail,malloc memory for pool or pool->list is error;
	*/
	int threadpool_init(int size);

	/*
	* run the function with the thread from pool
	* parameter:
	* 				func:the thread callback function
	* 				arg:the parameter of callback function
	* return:
	* 				0 : success
	* 				-1: the pool is NULL;
	* 				-2 : malloc memory for thread is error;
	* 				-3 : create thread is error;
	*/
	int threadpool_run(callback func, void *arg);

	/*
	* free and destroy the thread pool memory
	* return:
	* 				0 : success
	* 				less 0 : fail
	*/
	int threadpool_destroy();

#ifdef __cplusplus
}
#endif

#endif /* PTHREAD_POOL_H_ */



//
///*
//* Copyright (c) 2016, Mathias Brossard <mathias@brossard.org>.
//* All rights reserved.
//*
//* Redistribution and use in source and binary forms, with or without
//* modification, are permitted provided that the following conditions are
//* met:
//*
//*  1. Redistributions of source code must retain the above copyright
//*     notice, this list of conditions and the following disclaimer.
//*
//*  2. Redistributions in binary form must reproduce the above copyright
//*     notice, this list of conditions and the following disclaimer in the
//*     documentation and/or other materials provided with the distribution.
//*
//* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//* HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//*/
//
//#ifndef _THREADPOOL_H_
//#define _THREADPOOL_H_
//
//#ifdef __cplusplus
//extern "C" {
//#endif
//
//	/**
//	* @file threadpool.h
//	* @brief Threadpool Header File
//	*/
//
//	/**
//	* Increase this constants at your own risk
//	* Large values might slow down your system
//	*/
//#define MAX_THREADS 64
//#define MAX_QUEUE 65536
//
//	typedef struct threadpool_t threadpool_t;
//
//	typedef enum {
//		threadpool_invalid = -1,
//		threadpool_lock_failure = -2,
//		threadpool_queue_full = -3,
//		threadpool_shutdown = -4,
//		threadpool_thread_failure = -5
//	} threadpool_error_t;
//
//	typedef enum {
//		threadpool_graceful = 1
//	} threadpool_destroy_flags_t;
//
//	/**
//	* @function threadpool_create
//	* @brief Creates a threadpool_t object.
//	* @param thread_count Number of worker threads.
//	* @param queue_size   Size of the queue.
//	* @param flags        Unused parameter.
//	* @return a newly created thread pool or NULL
//	*/
//	threadpool_t *threadpool_create(int thread_count, int queue_size, int flags);
//
//	/**
//	* @function threadpool_add
//	* @brief add a new task in the queue of a thread pool
//	* @param pool     Thread pool to which add the task.
//	* @param function Pointer to the function that will perform the task.
//	* @param argument Argument to be passed to the function.
//	* @param flags    Unused parameter.
//	* @return 0 if all goes well, negative values in case of error (@see
//	* threadpool_error_t for codes).
//	*/
//	int threadpool_add(threadpool_t *pool, void(*routine)(void *),
//		void *arg, int flags);
//
//	/**
//	* @function threadpool_destroy
//	* @brief Stops and destroys a thread pool.
//	* @param pool  Thread pool to destroy.
//	* @param flags Flags for shutdown
//	*
//	* Known values for flags are 0 (default) and threadpool_graceful in
//	* which case the thread pool doesn't accept any new tasks but
//	* processes all pending tasks before shutdown.
//	*/
//	int threadpool_destroy(threadpool_t *pool, int flags);
//
//#ifdef __cplusplus
//}
//#endif
//
//#endif /* _THREADPOOL_H_ */